@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2011 The OpenLDAP Foundation.
+ * Copyright 2001-2016 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -124,18 +124,19 @@ monitor_entry_test_flags LDAP_P((
 	monitor_entry_t		*mp,
 	int			cond ));
 extern monitor_entry_t *
-monitor_entrypriv_create LDAP_P((
+monitor_back_entrypriv_create LDAP_P((
 	void ));
-
 extern Entry *
-monitor_entry_stub LDAP_P((
+monitor_back_entry_stub LDAP_P((
 	struct berval	*pdn,
 	struct berval	*pndn,
 	struct berval	*rdn,
 	ObjectClass		*oc,
-	monitor_info_t	*mi,
 	struct berval	*create,
-	struct berval	*modify));
+	struct berval	*modify ));
+
+#define monitor_entrypriv_create monitor_back_entrypriv_create
+#define monitor_entry_stub monitor_back_entry_stub
 
 /*
  * init
@@ -147,6 +148,9 @@ extern int
 monitor_back_register_subsys LDAP_P((
 	monitor_subsys_t	*ms ));
 extern int
+monitor_back_register_subsys_late LDAP_P((
+	monitor_subsys_t	*ms ));
+extern int
 monitor_back_register_backend LDAP_P((
 	BackendInfo		*bi ));
 extern int
