@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2015 The OpenLDAP Foundation.
+ * Copyright 1998-2016 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -741,6 +741,9 @@ static void connection_abandon( Connection *c )
 		SlapReply rs = {REP_RESULT};
 
 		next = LDAP_STAILQ_NEXT( o, o_next );
+		/* don't abandon an op twice */
+		if ( o->o_abandon )
+			continue;
 		op.orn_msgid = o->o_msgid;
 		o->o_abandon = 1;
 		op.o_bd = frontendDB;
